#include <stdio.h>

#include "brd.h"

#include "f4_gpio.h"
#include "f4_usart.h"
//#include "f4_timer.h"
#include "ols_fs.h"
#include "ols_drv_null.h"
#include "ols_drv_usart.h"
//#include "ols_drv_vcp.h"
#include "serial.h"
//#include "stm32f4xx_hal_tim.h"
#ifdef BRD_LED_TIMER
#include "tim.h"
#endif

unsigned int brd_buttons;
char *brd_stdio= BRD_STDIO;
enum brd_lmode_t brd_led_mode= BRD_LMODE_NONE;

void
brd_init()
{
  char s[10];
  unsigned int b, m;

#ifdef BRD_USART
#ifdef BRD_TX_PIN
  usart_devices[BRD_USART].tx_pin= BRD_TX_PIN;
#endif  
#ifdef BRD_RX_PIN
  usart_devices[BRD_USART].rx_pin= BRD_RX_PIN;
#endif  
#endif
  brd_type_init();
  ols_drv_null_install();
  ols_drv_usart_install();
  ols_stdio_init(brd_stdio, brd_stdio, brd_stdio);
  tc_setsparams(0, "115200,N,8,1");
  setvbuf(stdout, NULL, _IONBF, 0);
  write(0, "\a", 1);
  
  for (b= 0, m= 1; m < 0x10000; m<<= 1, b++)
    {
      if (brd_leds & m)
	{
	  snprintf(s, 9, "%s%d", f4_gpio_name(brd_led_port), b);
	  f4_gpio_output(s);
	  brd_led_off(m);
	}
      if (brd_btns & m)
	{
	  snprintf(s, 9, "%s%d", f4_gpio_name(brd_btn_port), b);
	  f4_gpio_pin_init_of(s,
#ifdef brd_btn_mode
			      brd_btn_mode
#else
			      "IP2N"
#endif
			      );
	}
    }
  brd_sample_buttons();
  brd_led_mode= BRD_LMODE_LED;
}

void
brd_set_led_mode(enum brd_led led, enum brd_lmode_t lmode)
{
#ifndef BRD_LED_TIMER
  lmode= BRD_LMODE_LED;
#endif
	if (lmode != brd_led_mode)
	{
		char s[100];
		int b= 0;
		while ((led & (1<<b)) == 0) b++;
		snprintf(s, 99, "%s%d", f4_gpio_name(brd_led_port), b);
		if (lmode == BRD_LMODE_LED)
		{
			f4_gpio_output(s);
			brd_led_mode= lmode;
		}
#ifdef BRD_LED_TIMER
		else if (led == LED0)
		{
			f4_gpio_af(s, BRD_LED_PWM_AF);
			brd_led_mode= lmode;
		}
#endif
	}
}

void
brd_led_on(enum brd_led led)
{
	//brd_led_port->BSRR= led;
	brd_led_port->ODR_NAME |= led;
	brd_set_led_mode(led, BRD_LMODE_LED);
}

void
brd_led_off(enum brd_led led)
{
	//brd_led_port->BSRR= (led<<16);
	brd_led_port->ODR_NAME &= ~led;
	brd_set_led_mode(led, BRD_LMODE_LED);
}

void
brd_led_toggle(enum brd_led led)
{
	brd_led_port->ODR_NAME ^= led;
	brd_set_led_mode(led, BRD_LMODE_LED);
}

void
brd_led_set(enum brd_led led, int on)
{
  if (on)
    //brd_led_port->BSRR= led;
	  brd_led_port->ODR_NAME |= led;
  else
    //brd_led_port->BSRR= (led<<16);
	  brd_led_port->ODR_NAME &= ~led;
	brd_set_led_mode(led, BRD_LMODE_LED);
}

int
brd_led_get(enum brd_led led)
{
	return (brd_led_port->ODR_NAME & led)?1:0;
}

uint32_t
brd_led_pwm(float power)
{
#ifdef BRD_LED_TIMER
	uint32_t arr, cl;
	float d;
	brd_set_led_mode(LED0, BRD_LMODE_PWM);
	arr= BRD_LED_TIMER.Instance->BRD_ARR_NAME;
	d= 0.01 * power * power;
	cl= arr*(d/100.0);
	BRD_LED_TIMER.Instance->BRD_CL_NAME= cl;
	return cl;
#else
	if (power > 50)
	  {
	    brd_led_set(LED0, 1);
	    return 1;
	  }
	else
	  {
	    brd_led_set(LED0, 0);
	    return 0;
	  }
#endif
}

void
brd_sample_buttons()
{
	static unsigned int last;
	unsigned int now= HAL_GetTick();
	if (now - last > 1000/50)
	{
		brd_buttons= brd_btn_port->IDR_NAME & (brd_btns);
		last= now;
	}
}

unsigned int
brd_button(enum brd_btn btn)
{
	if (brd_btn_active_state)
		return (brd_buttons & btn)?1:0;
	else
		return (brd_buttons & btn)?0:1;
}


static unsigned int brd_last_buttons;

int
brd_button_pressed(enum brd_btn btn)
{
	int a, l;
	a= brd_buttons & btn;
	l= brd_last_buttons & btn;
	if (a != l)
	{
		int ab= a?1:0;
		brd_last_buttons&= ~btn;
		brd_last_buttons|= a;
		if ((brd_btn_active_state && ab) ||
			(!brd_btn_active_state && !ab))
		{
			return 1;
		}
	}
	return 0;
}


/* End of brd.c */
